
#ifndef __ALGORITHMAN__BINARYTREE_CPP
    #define __ALGORITHMAN__BINARYTREE_CPP

    namespace AlgorithMan
    {

        template<class T> BinaryTreeNode<T>::BinaryTreeNode(T cont)
        {
            lsucc = NULL;
            rsucc = NULL;
            content = cont;
        }
    
        template<class T> BinaryTreeNode<T>::~BinaryTreeNode()
        {
            if(lsucc != NULL)
                delete lsucc;
            if(rsucc != NULL)
                delete rsucc;
        }

        template<class T> typename BinaryTreeNode<T>::iterator BinaryTreeNode<T>::end()
        {
            return NULL;
        }

        template<class T> BinaryTreeNode<T>* BinaryTreeNode<T>::Clone() const
        {
            BinaryTreeNode<T>* result = new BinaryTreeNode<T>(content);
            result->lsucc = lsucc == NULL ? NULL : lsucc->Clone();
            result->rsucc = rsucc == NULL ? NULL : rsucc->Clone();
        }

    
    
    
    
        template<class T> BinaryTree<T>::BinaryTree()
        {
            root = NULL;
        }
    
        template<class T> BinaryTree<T>::BinaryTree(const BinaryTree<T>& cpy)
        {
            root = cpy->root == NULL ? NULL : cpy->root->Clone();
        }

        // move constructor
        template<class T> BinaryTree<T>::BinaryTree(BinaryTree<T>&& mov)
        {
            root = mov->root;
            mov->root = NULL;
        }

        template<class T> BinaryTree<T>& BinaryTree<T>::operator=(const BinaryTree<T>& cpy)
        {
            clear();
            root = cpy->root == NULL ? NULL : cpy->root->Clone();
            return *this;
        }
    
        template<class T> BinaryTree<T>::~BinaryTree()
        {
            clear();
        }
    
        template<class T> void BinaryTree<T>::clear()
        {
            if(root != NULL)
                delete root;
            root = NULL;
        }
    
        template<class T> typename BinaryTree<T>::iterator BinaryTree<T>::end()
        {
            return NULL;
        }
    
    
    
    
    
        template<class T> typename BinaryTree<T>::iterator BinaryTree<T>::find(T value) const
        {
            for(typename BinaryTreeNode<T>::iterator i = root; i != NULL;)
            {
                if(value < i->content)
                    i = i->lsucc;
                else if(value > i->content)
                    i = i->rsucc;
                else return i;
            }
        
            return NULL;
        }
    
    
    
    
    
        template<class T> typename BinaryTree<T>::iterator BinaryTree<T>::insert(T value)
        {
            if(root == NULL)
                return root = new BinaryTreeNode<T>(value);
        
            typename BinaryTreeNode<T>::iterator i = root;
            while(true)
            {
                if(value < i->content)
                {
                    if(i->lsucc == NULL)
                        return i->lsucc = new BinaryTreeNode<T>(value);
                    i = i->lsucc;
                }
                else if(value > i->content)
                {
                    if(i->rsucc == NULL)
                        return i->rsucc = new BinaryTreeNode<T>(value);
                    i = i->rsucc;
                }
                else
                    return i; // already in the tree
            }
        }


    
    
    
        template<class T> void BinaryTreeNode<T>::output(std::ostream& os, int depth) const
        {
            if(lsucc != NULL)
                lsucc->output(os, depth+1);
            os << std::string(2*depth, ' ') << content << "\n";
            if(rsucc != NULL)
                rsucc->output(os, depth+1);
        }
    
        template<class T> void BinaryTree<T>::output(std::ostream& os) const
        {
            if(root != NULL)
                root->output(os, 0);
        }

        
    
    
    
        template<class T> int BinaryTreeNode<T>::depth() const
        {
            int r = rsucc == NULL ? 0 : rsucc->depth();
            int l = lsucc == NULL ? 0 : lsucc->depth();
            return l > r ? l+1 : r+1;
        }
    
        template<class T> int BinaryTree<T>::depth() const
        {
            return root == NULL ? 0 : root->depth();
        }
    
        template<class T> int BinaryTreeNode<T>::leaves() const
        {
            int r = rsucc == NULL ? 0 : rsucc->leaves();
            int l = lsucc == NULL ? 0 : lsucc->leaves();
            return l + r == 0 ? 1 : r + l;
        }

        template<class T> int BinaryTree<T>::leaves() const
        {
            return root == NULL ? 0 : root->leaves();
        }
    
        template<class T> int BinaryTreeNode<T>::nodes() const
        {
            int r = rsucc == NULL ? 0 : rsucc->nodes();
            int l = lsucc == NULL ? 0 : lsucc->nodes();
            return r + l + 1;
        }
    
        template<class T> int BinaryTree<T>::nodes() const
        {
            return root == NULL ? 0 : root->nodes();
        }

    }

#endif